/*
 * Copyright 2003 Chris Morgan <cmorgan@alum.wpi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _H_JACK_OUT_H
#define _H_JACK_OUT_H

#include <jack/jack.h>

#define bool long

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


#define ERR_SUCCESS                    0
#define ERR_OPENING_JACK               1
#define ERR_RATE_MISMATCH              2
#define ERR_BYTES_PER_FRAME_INVALID    3


enum status_enum { PLAYING, PAUSED, STOPPED, CLOSED, RESET };
enum pos_enum { BYTES, MILLISECONDS };

#define PLAYED 1          /* played out of the speakers(estimated value but should be close */
#define WRITTEN_TO_JACK 2 /* amount written out to jack */
#define WRITTEN 3         /* amount written to the bio2jack device */

/**********************/
/* External functions */
void JACK_Init(void);          /* initialize the jack library, this needs to be called before all other functions */
int  JACK_Open(int* deviceID, unsigned int bits_per_sample, unsigned long *rate, int channels);
int  JACK_Close(int deviceID); /* return 0 for success */
void JACK_Reset(int deviceID); /* free all buffered data and reset several values in the device */
long JACK_Write(int deviceID, char *data, unsigned long bytes); /* returns the number of bytes written */

/* state setting values */
/* set/get the written/played/buffered value based on a byte or millisecond input value */
long JACK_GetPosition(int deviceID, enum pos_enum position, int type);
void JACK_SetPosition(int deviceID, enum pos_enum position, long value);

long JACK_GetJackLatency(int deviceID); /* return the latency in milliseconds of jack */

int JACK_SetState(int deviceID, enum status_enum state); /* playing, paused, stopped */
enum status_enum JACK_GetState(int deviceID);

long JACK_GetMaxBufferedBytes(int deviceID);
void JACK_SetMaxBufferedBytes(int deviceID, long max_buffered_bytes); /* set the max number of bytes the jack driver should buffer */


/* Properties of the jack driver */
int  JACK_SetVolume(int deviceID, int left, int right); /* returns 0 on success */
void JACK_GetVolume(int deviceID, int *left, int *right);

long JACK_GetBytesPerSecond(int deviceID); /* bytes_per_frame * sample_rate */
long JACK_GetBytesStored(int deviceID); /* bytes currently buffered in the device */
long JACK_GetBytesFreeSpace(int deviceID); /* bytes of free space in the buffers */
long JACK_GetBytesPerFrame(int deviceID);
int  JACK_GetNumChannels(int deviceID);
long JACK_GetSampleRate(int deviceID); /* samples per second */

#endif /* #ifndef JACK_OUT_H */

